@@ -19,7 +19,7 @@ build_time_vars = {'ABIFLAGS': '',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'aarch64-unknown-openbsd7.4',
+ 'BUILD_GNU_TYPE': 'aarch64-unknown-openbsd7.5',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
@@ -42,21 +42,20 @@ build_time_vars = {'ABIFLAGS': '',
                 "-I/usr/local/include/tk8.6 -I/usr/X11R6/include' "
                 "'--with-tcltk-libs=-L/usr/X11R6/lib -ltcl86 -ltk86' "
                 "'--enable-loadable-sqlite-extensions' '--with-lto' "
-                "'--enable-optimizations' '--prefix=/usr/local' "
-                "'--sysconfdir=/etc' '--mandir=/usr/local/man' "
-                "'--infodir=/usr/local/info' '--localstatedir=/var' "
-                "'--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' "
-                "'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib/' "
-                "'CPPFLAGS=-I/usr/local/include' "
+                "'--prefix=/usr/local' '--sysconfdir=/etc' "
+                "'--mandir=/usr/local/man' '--infodir=/usr/local/info' "
+                "'--localstatedir=/var' '--disable-silent-rules' "
+                "'--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe -g' "
+                "'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include' "
                 "'TCLTK_CFLAGS=-I/usr/local/include/tcl8.6 "
                 "-I/usr/local/include/tk8.6 -I/usr/X11R6/include' "
                 "'TCLTK_LIBS=-L/usr/X11R6/lib -ltcl86 -ltk86 -lpthread'",
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.11.5/Python-3.11.5/coverage.info',
+ 'COVERAGE_INFO': '/usr/obj/ports/Python-3.11.8/Python-3.11.8/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.11.5/Python-3.11.5/lcov-report',
+ 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.11.8/Python-3.11.8/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
@@ -65,6 +64,7 @@ build_time_vars = {'ABIFLAGS': '',
  'DECIMAL_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_64=1 -DANSI=1 '
                    '-DHAVE_UINT128_T=1',
  'DECIMAL_LDFLAGS': '-lm Modules/_decimal/libmpdec/libmpdec.a',
+ 'DEEPFREEZE_C': 'Python/deepfreeze/deepfreeze.c',
  'DEEPFREEZE_DEPS': './Tools/scripts/deepfreeze.py _bootstrap_python '
                     './Programs/_freeze_module.py \\',
  'DEEPFREEZE_OBJS': 'Python/deepfreeze/deepfreeze.o',
@@ -162,7 +162,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_CURSES_H': 1,
  'HAVE_CURSES_HAS_KEY': 1,
  'HAVE_CURSES_IMMEDOK': 1,
- 'HAVE_CURSES_IS_PAD': 0,
+ 'HAVE_CURSES_IS_PAD': 1,
  'HAVE_CURSES_IS_TERM_RESIZED': 1,
  'HAVE_CURSES_RESIZETERM': 1,
  'HAVE_CURSES_RESIZE_TERM': 1,
@@ -414,6 +414,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_RENAMEAT': 1,
  'HAVE_RL_APPEND_HISTORY': 1,
  'HAVE_RL_CATCH_SIGNAL': 1,
+ 'HAVE_RL_COMPDISP_FUNC_T': 1,
  'HAVE_RL_COMPLETION_APPEND_CHARACTER': 1,
  'HAVE_RL_COMPLETION_DISPLAY_MATCHES_HOOK': 1,
  'HAVE_RL_COMPLETION_MATCHES': 1,
@@ -594,17 +595,17 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'aarch64-unknown-openbsd7.4',
+ 'HOST_GNU_TYPE': 'aarch64-unknown-openbsd7.5',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.11 '
                    '/usr/local/include/python3.11',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.11',
- 'INSTALL': '/usr/obj/ports/Python-3.11.5/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.11.5/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.11.5/bin/install -c  -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.11.5/bin/install -c -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.11.5/bin/install -c -m 755',
+ 'INSTALL': '/usr/obj/ports/Python-3.11.8/bin/install -c',
+ 'INSTALL_DATA': '/usr/obj/ports/Python-3.11.8/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.11.8/bin/install -c  -m 755',
+ 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.11.8/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/usr/obj/ports/Python-3.11.8/bin/install -c -m 755',
  'INSTSONAME': 'libpython3.11.so.0.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -621,8 +622,8 @@ build_time_vars = {'ABIFLAGS': '',
  'LIBEXPAT_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g '
                     '-flto -g -std=c11 -Werror=implicit-function-declaration '
                     '-fvisibility=hidden -I/usr/local/include '
-                    '-fprofile-instr-use=code.profclangd -I./Include/internal '
-                    '-I. -I./Include -I/usr/local/include -fPIC -fPIC',
+                    '-I./Include/internal -I. -I./Include -I/usr/local/include '
+                    '-fPIC -fPIC',
  'LIBEXPAT_HEADERS': '\\',
  'LIBEXPAT_OBJS': '\\',
  'LIBFFI_INCLUDEDIR': '/usr/local/include',
@@ -632,9 +633,8 @@ build_time_vars = {'ABIFLAGS': '',
                     '-DHAVE_UINT128_T=1 -DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe '
                     '-g -O2 -pipe -g -flto -g -std=c11 '
                     '-Werror=implicit-function-declaration -fvisibility=hidden '
-                    '-I/usr/local/include -fprofile-instr-use=code.profclangd '
-                    '-I./Include/internal -I. -I./Include -I/usr/local/include '
-                    '-fPIC -fPIC',
+                    '-I/usr/local/include -I./Include/internal -I. -I./Include '
+                    '-I/usr/local/include -fPIC -fPIC',
  'LIBMPDEC_HEADERS': '\\',
  'LIBMPDEC_OBJS': '\\',
  'LIBOBJDIR': 'Python/',
@@ -929,31 +929,30 @@ build_time_vars = {'ABIFLAGS': '',
                              '-pipe -g -flto -g -std=c11 '
                              '-Werror=implicit-function-declaration '
                              '-fvisibility=hidden -I/usr/local/include '
-                             '-fprofile-instr-use=code.profclangd '
                              '-I./Include/internal -I. -I./Include '
                              '-I/usr/local/include -fPIC '
                              '-DPy_BUILD_CORE_BUILTIN',
  'PY_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g',
  'PY_CFLAGS_NODIST': '-flto -g -std=c11 -Werror=implicit-function-declaration '
                      '-fvisibility=hidden -I/usr/local/include '
-                     '-fprofile-instr-use=code.profclangd -I./Include/internal',
+                     '-I./Include/internal',
  'PY_COERCE_C_LOCALE': 1,
  'PY_CORE_CFLAGS': '-DNDEBUG -O2 -pipe -g  -fPIC -O2 -pipe -g -O2 -pipe -g '
                    '-flto -g -std=c11 -Werror=implicit-function-declaration '
                    '-fvisibility=hidden -I/usr/local/include '
-                   '-fprofile-instr-use=code.profclangd -I./Include/internal '
-                   '-I. -I./Include -I/usr/local/include -fPIC -DPy_BUILD_CORE',
+                   '-I./Include/internal -I. -I./Include -I/usr/local/include '
+                   '-fPIC -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L/usr/local/lib/ -flto -g '
-                    '-L/usr/obj/ports/Python-3.11.5/Python-3.11.5 '
+                    '-L/usr/obj/ports/Python-3.11.8/Python-3.11.8 '
                     '-L/usr/local/lib/',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/local/include',
  'PY_ENABLE_SHARED': 1,
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_LDFLAGS': '-L/usr/local/lib/',
- 'PY_LDFLAGS_NODIST': '-flto -g -L/usr/obj/ports/Python-3.11.5/Python-3.11.5 '
+ 'PY_LDFLAGS_NODIST': '-flto -g -L/usr/obj/ports/Python-3.11.8/Python-3.11.8 '
                       '-L/usr/local/lib/',
  'PY_LDFLAGS_NOLTO': '-L/usr/local/lib/ -flto=thin '
-                     '-L/usr/obj/ports/Python-3.11.5/Python-3.11.5 '
+                     '-L/usr/obj/ports/Python-3.11.8/Python-3.11.8 '
                      '-L/usr/local/lib/',
  'PY_SQLITE_ENABLE_LOAD_EXTENSION': 1,
  'PY_SQLITE_HAVE_SERIALIZE': 1,
@@ -963,7 +962,6 @@ build_time_vars = {'ABIFLAGS': '',
                         '-g -flto -g -std=c11 '
                         '-Werror=implicit-function-declaration '
                         '-fvisibility=hidden -I/usr/local/include '
-                        '-fprofile-instr-use=code.profclangd '
                         '-I./Include/internal -I. -I./Include '
                         '-I/usr/local/include -fPIC',
  'PY_SUPPORT_TIER': 0,
@@ -971,12 +969,13 @@ build_time_vars = {'ABIFLAGS': '',
  'Py_ENABLE_SHARED': 1,
  'Py_HASH_ALGORITHM': 0,
  'Py_STATS': 0,
+ 'Py_SUNOS_VERSION': 0,
  'Py_TRACE_REFS': 0,
  'QUICKTESTOPTS': '-x test_subprocess test_io test_lib2to3 \\',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.5/Python-3.11.5',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.8/Python-3.11.8',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -1020,10 +1019,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.5/Python-3.11.5 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.8/Python-3.11.8 '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.5/Python-3.11.5 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.11.8/Python-3.11.8 '
                './python -E ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -1054,8 +1053,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.11.5/Python-3.11.5',
- 'abs_srcdir': '/usr/obj/ports/Python-3.11.5/Python-3.11.5',
+ 'abs_builddir': '/usr/obj/ports/Python-3.11.8/Python-3.11.8',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.11.8/Python-3.11.8',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
